//I2C-Toolbox

// condition;		condition (0=Off, 1=On)
// output;		selected output (1-4 are "ON, if over-temperature"; 5-8 are "ON, if under-temperature")
// output_state;	state of the selected output (0=Off, 1=On)
// alarm;		enable or disable alarm (0=no alarm, 1=alarm)   
// hysteresis;		hysteresis of the sensor (0-9 C)
// i2c_address;		8 Bit to store the i2c-address of the sensor
// target_value;	signed 16 bit for the target temperature
// actual_value; 	signed 16 bit for the current temperature


//switch the output on/off
//check, if an alarm needs to occur
//state could be 0=off, 1=on or 10=alarm
void switch_i2c_output(tSensor *sensor, uint8_t state)
{
	static uint8_t iPCFOutput=0;
	uint8_t iRet=0;
		
	//check connection to PCF
	iRet = i2c_start(PCF_ADDR+I2C_WRITE);
	if (iRet==true)
	{
		i2c_stop();
		return;
	}
	
	//check for an alarm-condition
	if(state == 10)
	{
		LEDPORT &= (uint8_t)~(1<<LED_RED);
		iPCFOutput |= OUT_ALARM;
	}
	
	//is the number of the output in the range 1-4? (this is the normal behaviour)
	if((sensor->output)<4)
	{		
		//what is the state we should switch to?
		if(state == true)					//if we should switch to "on"
		{
			iPCFOutput |= (uint8_t)(1<<(sensor->output));	//enable output
			(sensor->output_state)=1;
		}
		
		if(state == false)
		{
			iPCFOutput &= (uint8_t)~(1<<(sensor->output));	//disable output
			(sensor->output_state)=0;
		}
	}

	//is the number of the output between 5-8, then substract by four and invert the switching behaviour
	else
	{
		//what is the state we should switch to?
		if(state == true)					//if we should switch to "on"
		{
			iPCFOutput &= (uint8_t)~(1<<((sensor->output)-4));	//disable output (inverted function!)
			(sensor->output_state)=0;
		}
		
		if(state == false)
		{
			iPCFOutput |= (uint8_t)(1<<((sensor->output)-4));	//enable output (inverted function!)
			(sensor->output_state)=1;
		}
	}
		
	i2c_write(iPCFOutput);						//write this value to PCF
	i2c_stop();
}

//read the value of the digital sensor
//decides, how to return the value read, by take a look at the address of the sensor
void get_digital_sensor_value(tSensor *sensor)
{
	uint8_t iTemp=0;
	int8_t iTemperature=0;
	
	iTemp = i2c_start((sensor->i2c_address)+I2C_READ);		//open I2C-connection to sensor
	if (iTemp==true)
	{
		i2c_stop();	
		(sensor->actual_value) = SENSOR_ERROR;			//return an error if failed
		return;
	}
	
	iTemperature = i2c_readAck();	//read high-byte
	iTemp = i2c_readNak();		//read low-byte
	i2c_stop();
	
	//check the address - if it is in the range of a LM75, then only return the high-byte
	if((sensor->i2c_address)>71 && (sensor->i2c_address)<159)	
		(sensor->actual_value) = iTemperature;

	//otherwise return high- and low-byte as word
	else
		(sensor->actual_value) = ((iTemperature << 8) | iTemp);
	
}

//here we check, if the current sensor-output needs to to be switched
//this check includes the hysteresis of the sensor
void check_i2c_sensor(tSensor *sensor)
{
	//check, if sensor is active
	if((sensor->condition)==false)		//if not, leave function
		return;
	
	//check for alarm-condition
	if((sensor->alarm)==true)
		if ((sensor->target_value)<(sensor->actual_value))	//if over temperature
			switch_i2c_output(sensor,10);			//than make noise
			
	//over-temperature (normally used for heating)
	if ((sensor->target_value)<((sensor->actual_value)-(sensor->hysteresis)))
	{
		//disable output (e.g. switch off heating-plate)
		switch_i2c_output(sensor,0);
		return;
	}
	
	//under-temperature (normally used for heating)
	if ((sensor->target_value)>((sensor->actual_value)+(sensor->hysteresis)))
	{
		//enable output (e.g. switch on heating-plate)
		switch_i2c_output(sensor,1);
		return;
	}
	
}
