/************************************************************************/
/*                                                                      */
/*                      Debouncing 8 Keys				*/
/*			Sampling 4 Times				*/
/*			With Repeat Function				*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/

//key-port-definitions
#define BUTTONNPIN	PINC
#define BUTTONNPORT	PORTC
#define BUTTON1		3
#define BUTTON1P		PC3
#define BUTTON2		2
#define BUTTON2P		PC2
#define BUTTON3		1
#define BUTTON3P		PC1
#define BUTTON4		0
#define BUTTON4P		PC0
#define REPEAT_MASK	(1<<BUTTON2^1<<BUTTON3)	// keys to repeat
#define REPEAT_START	100		// after 1000ms
#define REPEAT_NEXT	10		// every 100ms

//#include <io.h>
//#include <interrupt.h>
//#include <signal.h>

typedef unsigned char	u8;
typedef signed short	s16;

u8 key_state;				// debounced and inverted key state:
					// bit = 1: key pressed
u8 key_press;				// key press detect

u8 key_rpt;				// key long press and repeat


SIGNAL (SIG_OVERFLOW0)			// every 10ms
{
  static u8 ct0, ct1, rpt;
  u8 i;

  TCNT0 = (u8)(s16)-(XTAL / 1024 * 10e-3 + 0.5);	// preload for 10ms

  i = key_state ^ ~BUTTONNPIN;		// key changed ?
  ct0 = ~( ct0 & i );			// reset or count ct0
  ct1 = ct0 ^ (ct1 & i);		// reset or count ct1
  i &= ct0 & ct1;			// count until roll over ?
  key_state ^= i;			// then toggle debounced state
  key_press |= key_state & i;		// 0->1: key press detect

  if( (key_state & REPEAT_MASK) == 0 )	// check repeat function
     rpt = REPEAT_START;		// start delay
  if( --rpt == 0 ){
    rpt = REPEAT_NEXT;			// repeat delay
    key_rpt |= key_state & REPEAT_MASK;
  }
}


u8 get_key_press( u8 key_mask )
{
  cli();					// read and clear atomic !
  key_mask &= key_press;                        // read key(s)
  key_press ^= key_mask;                        // clear key(s)
  sei();
  return key_mask;
}


u8 get_key_rpt( u8 key_mask )
{
  cli();					// read and clear atomic !
  key_mask &= key_rpt;                        	// read key(s)
  key_rpt ^= key_mask;                        	// clear key(s)
  sei();
  return key_mask;
}


u8 get_key_short( u8 key_mask )
{
  cli();			// read key state and key press atomic !
  return get_key_press( ~key_state & key_mask );
}


u8 get_key_long( u8 key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}

void keyinit(void)
{
	//Timer0 initialisieren
	TCCR0 = 1<<CS02^1<<CS00;			// divide by 1024
  	TIMSK = 1<<TOIE0;				// enable timer interrupt
  	//Taster-Datenrichtung auf Eingang
  	BUTTONNPORT &= ~((1<<BUTTON1P)|(1<<BUTTON2P)|(1<<BUTTON3P)|(1<<BUTTON4P));
}
