//System-includes
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

//own includes
#include "main.h"
#include "inc/lcd.c"
#include "inc/key_in.c"
#include "inc/lcd_toolbox.c"
#include "inc/twimaster.c"
#include "inc/uart.c"
#include "inc/i2c_toolbox.c"

//EEPROM variables
//condition, output, hysteresis, i2c-address
uint8_t ee_data_state[6][4] EEMEM;
//target-temperature
int16_t ee_data_temp[6] EEMEM;

//Prototypes
void initialize(void);
void read_sensordata_from_eeprom(tSensor*, uint8_t);
void write_sensordata_to_eeprom(tSensor*, uint8_t);


int main(void)						//The main routine
{
	uint8_t iSensorNumber=0, counter=0;		//Sensor-number to cycle through
	
	//define and initialize the Sensors 
	//TODO: read these values from eeprom
	tSensor tSensor1={0,0,0,0,5,0x11,0,0}, tSensor2={0,0,0,0,5,0x12,0,0}, tSensor3={0,0,0,0,5,0x13,0,0}, \
		tSensor4={0,0,0,0,5,0x14,0,0}, tSensor5={1,4,0,0,2,0x9E,18,0}, tSensor6={1,4,0,0,2,0x92,18,0};

	initialize();					//initialize the hardware
	write_menus(10);				//initially print the main-menu to the screen
	
	while(true)					//from here we run into the main-loop
	{

	wdt_reset();					//reset watchdog

	if(get_key_press(1<<BUTTON1)) 			//If button 1 is pressed
		{
			while(true)			//run into the next loop
			{
				wdt_reset();		//reset watchdog
				
				iSensorNumber = write_choose_sensor();	//let the user select the sensor and return its number
	
				switch(iSensorNumber)		//Cycle through the sensors and change the settings
				{
					case 1:
					{
						write_settings_menu(&tSensor1);
						break;
					}
					case 2:
					{
						write_settings_menu(&tSensor2);
						break;
					}
					case 3:
					{
						write_settings_menu(&tSensor3);
						break;
					}
					case 4:
					{
						write_settings_menu(&tSensor4);
						break;
					}
					case 5:
					{
						write_settings_menu(&tSensor5);
						break;
					}
					case 6:
					{
						write_settings_menu(&tSensor6);
						break;
					}
				}
				
				if(iSensorNumber==0)
				{
					write_menus(10);
					break;
				}

			}
		}
				
		//this is the reset-routine
		if(get_key_press(1<<BUTTON4))	
		{
			lcd_clrscr();			//cleanup the screen
			write_menus(13);		//print the reset-menu

			while(true)			//and wait for key-press
			{
				wdt_reset();		//reset watchdog
				if(get_key_press(1<<BUTTON1)) 	//if yes is pressed, run into a loop, until watchdog bites
				{
					lcd_clrscr();
					while(1);
				}
				if(get_key_press(1<<BUTTON4))	//else, retun to normal operation
				{
					lcd_clrscr();
					write_menus(10);
					break;
				}
			}
			
		}
		
		//This block writes the current values to the lcd
		write_to_position(0,0,1,&tSensor1);
		write_to_position(0,11,2,&tSensor2);		
		write_to_position(1,0,3,&tSensor3);
		write_to_position(1,11,4,&tSensor4);
		write_to_position(2,0,5,&tSensor5);
		write_to_position(2,11,6,&tSensor6);

		
		//read all sensors after a number of turns
		if(counter == 50)
		{
			LEDPORT &= (uint8_t)~(1<<LED_GREEN);

	        	get_digital_sensor_value(&tSensor1);
	        	check_i2c_sensor(&tSensor1);
	        	get_digital_sensor_value(&tSensor2);
	        	check_i2c_sensor(&tSensor2);
	        	get_digital_sensor_value(&tSensor3);
	        	check_i2c_sensor(&tSensor3);
	        	get_digital_sensor_value(&tSensor4);
	        	check_i2c_sensor(&tSensor4);
	        	get_digital_sensor_value(&tSensor5);
	        	check_i2c_sensor(&tSensor5);
	        	get_digital_sensor_value(&tSensor6);
	        	check_i2c_sensor(&tSensor6);
	        	counter=0;	//reset counter

			LEDPORT |= (uint8_t)(1<<LED_GREEN);	        	
	        }
	        counter++;	//increase counter
	}
		
	return 0;	//Only for the compiler - we never reach this point
}


//Initialize the whole system
void initialize(void)
{
	uint8_t iRet=0;
	i2c_init();					//Initialize I2C in master-mode
	
	iRet = i2c_start(PCF_ADDR+I2C_WRITE);		//connect to PCF
	if (iRet==true)
		i2c_stop();
	else						//if connect succedes
	{
		i2c_write(0);				//switch all outputs to off
		i2c_stop();
	}

	LEDDDR |= ((1<<LED_RED)|(1<<LED_GREEN));	//Init the DDR for the LED's
	LEDPORT |= ((1<<LED_RED)|(1<<LED_GREEN));	//set ports to 1 (LED's are active-low)

	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,XTAL));	//Initialize USART
	keyinit();					//Initialize the key-input routine
	lcd_init(LCD_DISP_ON);				//Initialize LCD
	load_usergraphics();				//Load user-defined graphics
	lcd_clrscr();					//clear screen
	wdt_enable(WDTO_500MS);				//enable watchdog
	sei();						//enable interrupts
}

void read_sensordata_from_eeprom(tSensor *sensor, uint8_t sensornumber)
{
	
	(sensor->condition) = 	eeprom_read_byte(&ee_data_state[sensornumber][0]);
	(sensor->output) = 	eeprom_read_byte(&ee_data_state[sensornumber][1]);
	(sensor->hysteresis) = 	eeprom_read_byte(&ee_data_state[sensornumber][2]);
	(sensor->i2c_address) = eeprom_read_byte(&ee_data_state[sensornumber][3]);
	(sensor->target_value) = eeprom_read_word(&ee_data_temp[sensornumber]);
}

void write_sensordata_to_eeprom(tSensor *sensor, uint8_t sensornumber)
{
	eeprom_write_byte(&ee_data_state[sensornumber][0], (sensor->condition));
	eeprom_write_byte(&ee_data_state[sensornumber][1], (sensor->output));
	eeprom_write_byte(&ee_data_state[sensornumber][2], (sensor->hysteresis));
	eeprom_write_byte(&ee_data_state[sensornumber][3], (sensor->i2c_address));
	eeprom_write_word(&ee_data_temp[sensornumber], (sensor->target_value));
}
